class CoolDownTracker:
    """
    Keep track of cool down settings for a specific WHOIS server
    """

    def __init__(self, cool_down_length, max_requests_minute=None, max_requests_hour=None, max_requests_day=None):
        """
        Create a tracker. It can accept three maximums. When a maximum is reached, it will wait a set amount of time
        before trying again, which is a minute, hour and day respectively.
        :param cool_down_length: The default length of the cool down
        :param max_requests_minute: The maximum number of requests per minute.
        :param max_requests_hour: The maximum number of requests per hour
        :param max_requests_day: The maximum number of request per day
        """
        self.cool_down_length = cool_down_length
        self.max_requests_minute = max_requests_minute
        self.max_requests_hour = max_requests_hour
        self.max_requests_day = max_requests_day

        self.request_count = 0
        self.current_cool_down = 0

    def use_whois_server(self):
        """
        Tell the tracker that the corresponding server is going to be used.
        It will set the cool down, based on the amount of requests that already have been made
        """
        self.request_count += 1
        self.start_cool_down()

    def start_cool_down(self):
        """
        Start a new cool_down
        """
        if self.max_requests_reached(self.max_requests_day):
            self.current_cool_down = 86400
        elif self.max_requests_reached(self.max_requests_hour):
            self.current_cool_down = 3600
        elif self.max_requests_reached(self.max_requests_minute):
            self.current_cool_down = 60
        else:
            self.current_cool_down = self.cool_down_length

    def decrement_cool_down(self, seconds):
        """
        Decrement the current cooldown with the given value, implying
        that a given time has passed.
        :param seconds: The seconds to decrement the current cool down value with
        """
        self.current_cool_down -= seconds

    def max_requests_reached(self, limit):
        """
        Check whether the maximum has been reached for a given limit.
        :param limit: The limit that should be checked for
        :return: True if the limit has been reached, false if not
        """
        return limit is not None and self.request_count % limit == 0

    def double_cool_down(self):
        """
        Double the cool down length, as in, the cool down length that is always used,
        not the current cool down that happening.
        """
        self.cool_down_length *= 2
        self.start_cool_down()



import ast
import os


def read_cache(file_path):
    if os.path.isfile(file_path):
        return ast.literal_eval(open(file_path).read())
    else:
        if os.path.dirname(file_path):
            os.makedirs(os.path.dirname(file_path))
        return {}


def write_cache(cache, file_path):
    cache_file = open(file_path, 'w')
    cache_file.write(str(cache))


class WhoisServerCache:
    """
    Cache handler for easy of use. Do not instantiate. import server_cache instead.
    Otherwise an inconsistent cache can happen as a result of multiple caches.
    """

    def __init__(self):
        self.cache = {}
        self.persistent = False
        self.file_path = None

    def get_server(self, tld):
        """
        Get a WHOIS server for a given TLD
        :param tld: The TLD to get the WHOIS server for
        :return: The WHOIS server if it is known, or None otherwise
        """
        return self.cache.get(tld)

    def put_server(self, tld, whois_server):
        """
        Store a new WHOIS server in the cache. If the cache is persistent,
        it is also written to disk again. Because the WHOIS servers
        don't change that often, it simply writes to a file.
        :param tld: The TLD to store a WHOIS server for
        :param whois_server: The WHOIS server to store
        """
        self.cache[tld] = whois_server
        if self.file_path is not None:
            write_cache(self.cache, self.file_path)

    def set_persistent_location(self, file_path):
        """
        Store the cache in a persistent location
        :param file_path: The path to store the cache
        """
        self.file_path = file_path
        self.cache = read_cache(file_path)


server_cache = WhoisServerCache()



from . import net, parse


def get_whois(domain, normalized=[]):
    raw_data, server_list = net.get_whois_raw(domain, with_server_list=True)
    # Unlisted handles will be looked up on the last WHOIS server that was queried. This may be changed to also query
    # other servers in the future, if it turns out that there are cases where the last WHOIS server in the chain doesn't
    # actually hold the handle contact details, but another WHOIS server in the chain does.
    if len(server_list) > 0:
        handle_server = server_list[-1]
    else:
        handle_server = ""
    return parse.parse_raw_whois(raw_data, normalized=normalized, never_query_handles=False,
                                 handle_server=handle_server)


def set_persistent_cache(path_to_cache):
    """
    Set a persistent cache. If the file does not yet exist, it is created.
    :param path_to_cache: The place where the cache is stored or needs to be created
    """
    net.server_cache.set_persistent_location(path_to_cache)


def set_cool_down_config(path_to_config):
    """
    Set a cool down configuration file, describing specific settings for certain WHOIS servers.
    :param path_to_config: The path to the configuration file, this needs to exist
    """
    net.cool_down_tracker.set_cool_down_config(path_to_config)
